#pragma once

#include <math.h>
#include <iostream>

#include "Renderer.hpp"
#define STB_IMAGE_WRITE_IMPLEMENTATION
#include "stb_image_write.hpp"

namespace massivedynamic {
	typedef uint32_t Color;

	class PixelRenderer : public Renderer {
		protected:
		 	size_t pixelSize;
			std::vector<Color> bitmap;

			void drawPixelScaled(size_t x, size_t y, Color color) {
				size_t pixelSize = this->pixelSize;

				if (pixelSize < 1) {
					std::cerr << "ERROR: output file size is too small" << std::endl;
					exit(1);
				}

				size_t absoluteX = pixelSize * x + pixelSize;
				size_t absoluteY = pixelSize * y + pixelSize;

				for (size_t localY = absoluteY; localY < absoluteY + pixelSize; localY++) {
					for (size_t localX = absoluteX; localX < absoluteX + pixelSize; localX++) {
						this->bitmap.at(localY * this->targetSize + localX) = color;
					}
				}
			}
		public:
			enum Colors : Color {
				BLACK = 0xFF000000,
				WHITE = 0xFFFFFFFF
			};

			PixelRenderer(const std::vector<bool>& pixels, size_t sourceSize, size_t targetSize) : Renderer(pixels, sourceSize, targetSize), pixelSize(0) {
				this->targetSize = this->targetSize == 0 ? (sourceSize + 2) * 2 : this->targetSize;

				// here we make shure, `targetSize` will be a multiple of `sourceSize`	
				this->pixelSize = round(static_cast<float>(this->targetSize) / static_cast<float>(this->sourceSize + 2));
				this->targetSize = (this->sourceSize + 2) * this->pixelSize;

				this->bitmap = std::vector<Color>(this->targetSize * this->targetSize, Colors::WHITE);
			}

			void generateBuffer() {
				for (size_t y = 0; y < sourceSize; y++) {
					for (size_t x = 0; x < sourceSize; x++) {
						if (!pixels.at(y * sourceSize + x))
							continue;
						this->drawPixelScaled(x, y, Colors::BLACK);
					}
				}

			}
	};

}
